const showLog = false;
let os = '';
function log() {
    showLog && console.log.apply(console, arguments);
}

// note this gets called on extension install + update, but also on chrome update.
function recordInstall(currentAppVersion, currentTime) {

    // They may have existing values. So, we request them.
    chrome.storage.sync.get({
        "appVersionHistory": [],
        "initialAppInstallTime": currentTime
    }, savedVals => {

        // Add the current version to the user's history, if not already there.
        const versions = savedVals.appVersionHistory;
        if (!versions.includes(currentAppVersion)) {
            versions.push(currentAppVersion);
        }

        // Now we write the values back, likely setting them for the first time.
        chrome.storage.sync.set({
            "appVersionHistory": versions,
            "initialAppInstallTime": savedVals.initialAppInstallTime,
            "mostRecentUpdateTime": currentTime,
        }, updateUninstallUrl);

    });
}

// max url length is 255 chars, and we actually hit this limit sometimes.
// So, we'll try to build the longest url we can, giving preference for args at the front of the array.
function buildUrlRespectingMaxLength(queryArgs) {
    let url = 'http://rehfeld.us/browser-extensions/uninstall-survey.php?';

    let argSep = '';
    for (const arg of queryArgs) {
        if (url.length + argSep.length + arg.length < 255) {
            url += argSep + arg;
            // Trick to prevent adding & on first loop iteration.
            argSep = '&';
        }
    }

    return url;
}

function updateUninstallUrl() {
    chrome.storage.sync.get(["appVersionHistory", "initialAppInstallTime", "mostRecentUpdateTime"], syncVals => {
        chrome.storage.local.get({usageStats: {}}, localVals => {
            const manifest = chrome.runtime.getManifest();
            const usageStatsQueryArgs = Object.keys(localVals.usageStats).map(key => `${encodeURIComponent(key)}=${encodeURIComponent(localVals.usageStats[key])}`);

            // Add the current version to the user's history, if not already there.
            const versions = (syncVals.appVersionHistory || []);
            if (!versions.includes(manifest.version)) {
                versions.push(manifest.version);
            }
            const sortedVersions = versions.sort();

            // To save space, we only send min and max versions.
            let versionRange = sortedVersions[0];
            if (sortedVersions.length > 1) {
                versionRange +=  "," + sortedVersions[sortedVersions.length - 1];
            }

            // chrome imposes a 255 char url limit, so we need to use compact query var names.
            const queryArgs2 = {
                app: manifest.extNameCode,
                it: syncVals.initialAppInstallTime,
                ut: syncVals.mostRecentUpdateTime,
                vh: versionRange,
                // os might not yet have a value, but oh well.
                os: os,
            };

            const map = Object.assign({}, queryArgs2, localVals.usageStats);
            const urlKeyValPairs = Object.keys(map).map(key => {
                return encodeURIComponent(key) + '=' + encodeURIComponent(map[key]);
            })

            chrome.runtime.setUninstallURL(buildUrlRespectingMaxLength(urlKeyValPairs));
        });
    });
}

chrome.runtime.onInstalled.addListener(details => {
    log("onInstall", details);
    const manifest = chrome.runtime.getManifest();
    recordInstall(manifest.version, unixTimestamp());
});

// Record the os so the content-script can access it.
chrome.runtime.getPlatformInfo(function(info) {
    os = info.os;
});

chrome.storage.onChanged.addListener(updateUninstallUrl);

function unixTimestamp() {
    return Math.floor(Date.now() / 1000);
}