// Saves options to chrome.storage
function saveUserConfig() {
    chrome.storage.sync.set({
        "rememberVolumeLevel": byId("rememberVolumeLevel").checked,
        "playbackRate": byId("playbackRate").value
    }, () => {
        // Update status to let user know options were saved.
        const status = byId('status');
        status.classList.remove("hidden");
        status.classList.add("visible");
        status.textContent = "Config Saved.";
        setTimeout(() => {
            status.classList.remove("visible");
            status.classList.add("hidden");
        }, 200);
    });
}

function getSavedOptions() {
    return new Promise((resolve, reject) => {
        chrome.storage.sync.get({
            rememberVolumeLevel: true,
            playbackRate: 1
        }, resolve);
    });
}

function restoreUserConfig() {
    getSavedOptions().then(vals => {
        byId("rememberVolumeLevel").checked = vals["rememberVolumeLevel"];
        byId("playbackRate").value = vals["playbackRate"];
    });
}

function debounce(func, delayUntilCall) {
    let timerId;
    return function() {
        const that = this;
        const args = arguments;
        timerId && clearTimeout(timerId);
        timerId = setTimeout(function debounceWrapper() {
            timerId = undefined;
            func.apply(that, args);
        }, delayUntilCall);
    };
}

function byId(domId) {
    return document.getElementById(domId);
}

document.addEventListener('DOMContentLoaded', restoreUserConfig);

// Use input event on all elements to trigger a save.
// But, we debounce it, and only do the save if there's been no activity for a bit.
// This way we don't spam save as they type.
const markFormUpdated = debounce(saveUserConfig, 1000);
Array.from(document.querySelectorAll("input[type=number]")).forEach(input => {
    input.addEventListener("input", markFormUpdated, false);
});
Array.from(document.querySelectorAll("input[type=checkbox]")).forEach(input => {
    input.addEventListener("change", markFormUpdated, false);
});
